#!/usr/bin/env python
"""
A light-weight Table class for manipulating 2D data and representing it as text, or writing to file for import into other packages.

Current output formats include pickle (pythons serialisation format), restructured text (keyed by 'rest'), latex, html, delimited columns, and a simple text format.

Table can read pickled and delimited formats.

"""
from __future__ import division
import cPickle, csv
from gzip import GzipFile

import numpy
from cogent.format import table as table_format, bedgraph

from cogent.util.dict_array import DictArray

__author__ = "Gavin Huttley"
__copyright__ = "Copyright 2007-2016, The Cogent Project"
__credits__ = ["Gavin Huttley", "Felix Schill"]
__license__ = "GPL"
__version__ = "1.9"
__maintainer__ = "Gavin Huttley"
__email__ = "gavin.huttley@anu.edu.au"
__status__ = "Production"

# making reversed characters for use in reverse order sorting
_all_chrs = [chr(i) for i in range(256)]
_all_chrs.reverse()
_reversed_chrs = ''.join(_all_chrs)

def convert2DDict(twoDdict, header = None, row_order = None):
    """Returns a 2 dimensional list.
    
    Arguments:
        - twoDdict: a 2 dimensional dict with top level keys corresponding to
          column headings, lower level keys correspond to row headings but are
          not preserved.
        - header: series with column headings. If not provided, the sorted top
          level dict keys are used.
        - row_order: a specified order to generate the rows.
    """
    if not header:
        header = twoDdict.keys()
        header.sort()
    
    if not row_order: # we assume rows consistent across dict
        row_order = twoDdict[header[0]].keys()
        row_order.sort()
    
    # make twoD list
    table = []
    for row in row_order:
        string_row = []
        for column in header:
            string_row.append(twoDdict[column][row])
        table.append(string_row)
    return table

class _Header(list):
    """a convenience class for storing the Header"""
    def __new__(cls, arg):
        n = list.__new__(cls, list(arg))
        return n
    
    def __setslice__(self, *args):
        """disallowed"""
        raise RuntimeError("Table Header is immutable, use withNewHeader")
    
    def __setitem__(self, *args):
        """disallowed"""
        raise RuntimeError("Table Header is immutable, use withNewHeader")
    

class Table(DictArray):
    def __init__(self, header = None, rows = None, row_order = None, digits = 4,
                space = 4, title = '', missing_data = '', max_width = 1e100,
                row_ids = False, legend = '', column_templates = None,
                dtype = None):
        """
        Arguments:
        - header: column headings
        - rows: a 2D dict, list or tuple. If a dict, it must have column
          headings as top level keys, and common row labels as keys in each
          column.
        - row_order: the order in which rows will be pulled from the twoDdict
        - digits: floating point resolution
        - space: number of spaces between columns or a string
        - title: as implied
        - missing_data: character assigned if a row has no entry for a column
        - max_width: maximum column width for printing
        - row_ids: if True, the 0'th column is used as row identifiers and keys
          for slicing.
        - legend: table legend
        - column_templates: dict of column headings: string format templates
          or a function that will handle the formatting.
        - dtype: optional numpy array typecode.
        """
        try:
            num_cols = len(header)
            assert num_cols > 0
            if type(rows) == numpy.ndarray:
                assert num_cols == rows.shape[1]
            elif type(rows) == dict:
                assert num_cols == len(rows)
            else:
                assert num_cols == len(rows[0])
        except (IndexError, TypeError, AssertionError):
            raise RuntimeError("header and rows must be provided to Table")
        
        header = [str(head) for head in header]
        if isinstance(rows, dict):
            rows = convert2DDict(rows, header = header,
                                row_order = row_order)
        
        # if row_ids, we select that column as the row identifiers
        if row_ids:
            identifiers = [row[0] for row in rows]
        else:
            identifiers = len(rows)
        
        if not dtype:
            dtype = "O"
        DictArray.__init__(self, rows, identifiers, header, dtype = dtype)
        
        # forcing all column headings to be strings
        self._header = _Header([str(head) for head in header])
        self._missing_data = missing_data
        
        self.Title = str(title)
        self.Legend = str(legend)
        try:
            self.Space = ' ' * space
        except TypeError:
            self.Space = space
        self._digits = digits
        self._row_ids = row_ids
        self._max_width = max_width
        
        # some attributes are not preserved in any file format, so always based
        # on args
        self._column_templates = column_templates or {}
    
    def __repr__(self):
        row = []
        try:
            for val in self.array[0][:3]:
                if isinstance(val, float):
                    row.append('%.4f' % val)
                elif isinstance(val, int):
                    row.append('%d' % val)
                else:
                    row.append('%r' % val)
        except IndexError:
            # an empty table
            pass
        
        row_trunc = ', '.join(row)
        header_trunc = ', '.join(map(repr, self.Header[:3]))
        if self.Shape[1] > 3:
            header_trunc = '[%s,..]' % header_trunc
            row_trunc = '[%s,..]' % row_trunc
        else:
            header_trunc = '[%s]' % header_trunc
            row_trunc = '[%s]' % row_trunc
        
        if self.Shape[0] > 1:
            row_trunc = '[%s,..]' % row_trunc
        else:
            row_trunc = '[[%s]]' % row_trunc
        
        if self.Shape[0] == 0:
            row_trunc = str([])
        
        result = 'Table(numrows=%s, numcols=%s, header=%s, rows=%s)' % \
            (self.Shape[0], self.Shape[1], header_trunc, row_trunc)
        return result
    
    def __str__(self):
        return self.tostring()
    
    def __getitem__(self, names):
        (index, remaining) = self.template.interpretIndex(names)
        # if we have two integers, return a single value
        ints = [isinstance(idx, int) for idx in index]
        if len(ints) == 2 and min(ints):
            return self.array[index]
        new_index = list(index)
        for i, idx in enumerate(new_index):
            if isinstance(idx, int):
                new_index[i] = slice(idx, idx+1, None)
            
        index = tuple(new_index)
        rows = self.array[index]
        result = None
        if len(index) > 1:
            header = numpy.asarray(self.Header, dtype="O")[index[1:]]
        else:
            header = self.Header
        if remaining is not None:
            kwargs = self._get_persistent_attrs()
            result = self.__class__(header, rows, **kwargs)
        return result
    
    def __getstate__(self):
        data = self._get_persistent_attrs()
        del(data['column_templates'])
        data.update(dict(header = self.Header, rows = self.getRawData()))
        return data
    
    def __setstate__(self, data):
        limit_ids = data.pop('limit_ids', None)
        if limit_ids is not None:
            data['row_ids'] = limit_ids or False
        new = Table(**data)
        self.__dict__.update(new.__dict__)
        return self
    
    def _get_header(self):
        """returns Header value"""
        return self._header
    
    def _set_header(self, data):
        """disallowed"""
        raise RuntimeError("not allowed to set the Header")
    
    Header = property(_get_header, _set_header)
    
    def withNewHeader(self, old, new, **kwargs):
        """returns a new Table with old header labels replaced by new
        
        Arguments:
            - old: the old column header(s). Can be a string or series of
              them.
            - new: the new column header(s). Can be a string or series of
              them.
        """
        if type(old) == str:
            old = [old]
            new = [new]
        
        assert len(old) == len(new), 'Mismatched number of old/new labels'
        indices = map(self.Header.index, old)
        new_header = list(self.Header)
        for i in range(len(old)):
            new_header[indices[i]] = new[i]
        
        kw = self._get_persistent_attrs()
        kw.update(kwargs)
        return Table(header = new_header, rows = self.getRawData(), **kw)
    
    def _get_persistent_attrs(self):
        kws = dict(row_ids = self._row_ids, title = self.Title,
                   legend = self.Legend, digits = self._digits,
                   space = self.Space, max_width = self._max_width,
                   missing_data = self._missing_data,
                   column_templates = self._column_templates or None)
        return kws
    
    def setColumnFormat(self, column_head, format_template):
        """Provide a formatting template for a named column.
        
        Arguments:
            - column_head: the column label.
            - format_template: string formatting template or a function that
              will handle the formatting.
        """
        assert column_head in self.Header, \
               "Unknown column heading %s" % column_head
        
        self._column_templates[column_head] = format_template
    
    def tostring(self, borders=True, sep=None, format='', **kwargs):
        """Return the table as a formatted string.
        
        Arguments:
            - format: possible formats are 'rest', 'latex', 'html', 'phylip',
              'bedgraph', or simple text (default).
            - sep: A string separator for delineating columns, e.g. ',' or '\t'.
              Overrides format.
        
        NOTE: If format is bedgraph, assumes that column headers are chrom,
        start, end, value. In that order!
        """
        if format.lower() == 'phylip':
            missing_data = "%.4f" % 0.0
        else:
            missing_data = self._missing_data
        
        # convert self to a 2D list
        formatted_table = self.array.tolist()
        if format != 'bedgraph':
            header, formatted_table = table_format.formattedCells(formatted_table,
                                    self.Header,
                                    digits = self._digits,
                                    column_templates = self._column_templates,
                                    missing_data = missing_data)
            args = (header, formatted_table, self.Title, self.Legend)
        if sep and format != 'bedgraph':
            return table_format.separatorFormat(*args + (sep,))
        elif format == 'rest':
            return table_format.gridTableFormat(*args)
        elif format.endswith('tex'):
            caption = None
            if self.Title or self.Legend:
                caption = " ".join([self.Title or "", self.Legend or ""])
            return table_format.latex(formatted_table, header,
                                caption = caption, **kwargs)
        elif format == 'html':
            rest = table_format.gridTableFormat(*args)
            return table_format.html(rest)
        elif format == 'phylip':
            # need to eliminate row identifiers
            formatted_table = [row[self._row_ids:] for row in formatted_table]
            header = header[self._row_ids:]
            return table_format.phylipMatrix(formatted_table, header)
        elif format == 'bedgraph':
            assert self.Shape[1] == 4, 'bedgraph format is for 4 column tables'
            # assuming that header order is chrom, start, end, val
            formatted_table = bedgraph.bedgraph(self.sorted().array.tolist(),
                **kwargs)
            return formatted_table
        else:
            return table_format.simpleFormat(*args + (self._max_width,
                                self._row_ids, borders, self.Space))
    
    def toRichHtmlTable(self, row_cell_func=None, header_cell_func=None,
                element_formatters={}, merge_identical=True, compact=True):
        """returns just the table html code.
        Arguments:
            - row_cell_func: callback function that formats the row values. Must
              take the row value and coordinates (row index, column index).
            - header_cell_func: callback function that formats the column headings
              must take the header label value and coordinate
            - element_formatters: a dictionary of specific callback funcs for
              formatting individual html table elements.
              e.g. {'table': lambda x: '<table border="1" class="docutils">'}
            - merge_identical: cells within a row are merged to one span.
        """
        formatted_table = self.array.tolist()
        header, formatted_table = table_format.formattedCells(formatted_table,
                                    self.Header,
                                    digits = self._digits,
                                    column_templates = self._column_templates,
                                    missing_data = self._missing_data)
        # but we strip the cell spacing
        header = [v.strip() for v in header]
        rows = [[c.strip() for c in r] for r in formatted_table]
        return table_format.rich_html(rows, row_cell_func=row_cell_func,
                                      header=header,
                                      header_cell_func=header_cell_func,
                                      element_formatters=element_formatters,
                                      compact=compact)
    
    def writeToFile(self, filename, mode = 'w', writer = None, format = None,
                 sep = None, compress=None, **kwargs):
        """Write table to filename in the specified format. If a format is not
        specified, it attempts to use a filename suffix. Note if a sep argument
        is provided, unformatted values are written to file in order to preserve
        numerical accuracy.
        
        Arguments:
            - mode: file opening mode
            - format: Valid formats are those of the tostring method plus
              pickle.
            - writer: a function for formatting the data for output.
            - sep: a character delimiter for fields.
            - compress: if True, gzips the file and appends .gz to the
              filename (if not already added).
        """
        compress = compress or filename.endswith('.gz')
        
        if compress:
            if not filename.endswith('.gz'):
                filename = '%s.gz' % filename
            mode = ['wb', mode][mode == 'w']
            outfile = GzipFile(filename, mode)
        else:
            outfile = file(filename, mode)
        
        if format is None:
            # try guessing from filename suffix
            if compress:
                index = -2
            else:
                index = -1
            suffix = filename.split('.')
            if len(suffix) > 1:
                format = suffix[index]
        
        if writer:
            rows = self.getRawData()
            rows.insert(0, self.Header[:])
            rows = writer(rows, has_header=True)
            outfile.writelines("\n".join(rows))
        elif format == 'pickle':
            data = self.__getstate__()
            cPickle.dump(data, outfile)
        elif sep is not None and format != 'bedgraph':
            writer = csv.writer(outfile, delimiter = sep)
            if self.Title:
                writer.writerow([self.Title])
            writer.writerow(self.Header)
            writer.writerows(self.array)
            if self.Legend:
                writer.writerow([self.Legend])
        else:
            table = self.tostring(format = format, **kwargs)
            outfile.writelines(table + '\n')
        outfile.close()
    
    def toBedgraph(self, chrom_col, start_col, end_col):
        """docstring for toBedgraph"""
        pass
    
    def appended(self, new_column, *tables, **kwargs):
        """Append an arbitrary number of tables to the end of this one.
        Returns a new table object. Optional keyword arguments to the new
        tables constructor may be passed.
        
        Arguments:
            - new_column: provide a heading for the new column, each tables
            title will be placed in it. If value is false, the result is no
            additional column."""
        
        # convert series of tables
        if isinstance(tables[0], tuple) or isinstance(tables[0], list):
            tables = tuple(tables[0])
        # for each table, determine it's number of rows and create an equivalent
        # length vector of its title
        if new_column:
            header = [new_column] + self.Header
        else:
            header = self.Header
        
        big_twoD = ()
        table_series = (self,) + tables
        for table in table_series:
            # check compatible tables
            assert self.Header == table.Header, \
                   "Inconsistent tables -- column headings are not the same."
            new_twoD = []
            for row in table:
                if new_column:
                    new_twoD.append([table.Title] + row.asarray().tolist())
                else:
                    new_twoD.append(row.asarray().tolist())
            new_twoD = tuple(new_twoD)
            big_twoD += new_twoD
        kw = self._get_persistent_attrs()
        kw.update(kwargs)
        return Table(header, big_twoD, **kw)
    
    def getRawData(self, columns = None):
        """Returns raw data as a 1D or 2D list of rows from columns. If one
        column, its a 1D list.
        
        Arguments:
            - columns: if None, all data are returned"""
        
        if columns is None:
            return self.array.tolist()
        
        if isinstance(columns, str):
            # assumes all column headings are strings.
            columns = (columns,)
        
        column_indices = map(self.Header.index, columns)
        result = self.array.take(column_indices, axis=1)
        
        if len(columns) == 1:
            result = result.flatten()
        
        return result.tolist()
    
    def _callback(self, callback, row, columns=None, num_columns=None):
        if callable(callback):
            row_segment = row.take(columns)
            if num_columns == 1:
                row_segment = row_segment[0]
            return callback(row_segment)
        else:
            return eval(callback, {}, row)
    
    def filtered(self, callback, columns=None, **kwargs):
        """Returns a sub-table of rows for which the provided callback
        function returns True when passed row data from columns. Row data
        is a 1D list if more than one column, raw row[col] value otherwise.
        
        Arguments:
            - columns: the columns whose values determine whether a row is to
            be included.
            - callback: Can be a function, which takes the sub-row delimited
            by columns and returns True/False, or a string representing valid
            python code to be evaluated."""
        
        if isinstance(columns, str):
            columns = (columns,)
        
        if columns:
            num_columns = len(columns)
        else:
            num_columns = None
        
        row_indexes = []
        if not callable(callback):
            data = self
            cols = columns
        else:
            data = self.array
            cols = map(self.Header.index, columns)
        
        for rdex, row in enumerate(data):
            if self._callback(callback, row, cols, num_columns):
                row_indexes.append(rdex)
            
        sub_set = numpy.take(self, row_indexes, 0)
        
        kw = self._get_persistent_attrs()
        kw.update(kwargs)
        return Table(header = self.Header, rows = sub_set, **kw)
    
    def filteredByColumn(self, callback, **kwargs):
        """Returns a table with columns identified by callback
        
        Arguments:
            - callback: A function which takes the columns delimited
            by columns and returns True/False, or a string representing valid
            python code to be evaluated."""
        data = self.array.transpose()
        column_indices = []
        append = column_indices.append
        for index, row in enumerate(data):
            if callback(row):
                append(index)
        columns = numpy.take(self.Header, column_indices)
        return self.getColumns(columns, **kwargs)
    
    def count(self, callback, columns=None, **kwargs):
        """Returns number of rows for which the provided callback
        function returns True when passed row data from columns. Row data
        is a 1D list if more than one column, raw row[col] value otherwise.
        
        Arguments:
            - columns: the columns whose values determine whether a row is to
            be included.
            - callback: Can be a function, which takes the sub-row delimited
            by columns and returns True/False, or a string representing valid
            python code to be evaluated."""
        
        if isinstance(columns, str):
            columns = (columns,)
        
        if columns:
            num_columns = len(columns)
        else:
            num_columns = None
        
        count = 0
        if not callable(callback):
            data = self
            cols = columns
        else:
            data = self.array
            cols = map(self.Header.index, columns)
        
        for row in data:
            if self._callback(callback, row, cols, num_columns):
                count += 1
        return count
    
    def sorted(self, columns=None, reverse=None, **kwargs):
        """Returns a new table sorted according to columns order.
        
        If only reverse is provided, that order is used.
        
        Arguments:
            - columns: column headings, their order determines the sort order.
            - reverse: column headings, these columns will be reverse sorted.
            
            Either can be provided as just a single string, or a series of
            strings.
        """
        
        if reverse and columns is None:
            columns = reverse
        
        if columns is None:
            columns = self.Header
        elif isinstance(columns, str):
            columns = [columns]
        
        indices = [self.Header.index(col) for col in columns]
        
        if not reverse:
            is_reversed = [False] * len(columns)
            reverse_indices = []
        else:
            if type(reverse) == str:
                reverse = [reverse]
            reverse_indices = []
            for index, header_index in enumerate(indices):
                col = self.Header[header_index]
                if col in reverse:
                    reverse_indices += [index]
            
            is_reversed = [col in reverse for col in columns]
        
        reverse_indices = numpy.array(reverse_indices)
        
        dtypes = [(self.Header[i], self.array.dtype) for i in indices]
        
        # applying the decorate-sort-undecorate approach
        aux_list = self.array.take(indices, axis=1)
        
        # we figure out the casting funcs for any reversed elements
        cast = []
        for index in reverse_indices:
            val = aux_list[0, index]
            try:
                val = val.translate(_reversed_chrs)
                func = lambda x: x.translate(_reversed_chrs)
            except AttributeError:
                func = lambda x: x * -1
            func = numpy.vectorize(func)
            aux_list[:, index] = func(aux_list[:, index])
        
        aux_list = numpy.rec.fromarrays(aux_list.copy().T, dtype=dtypes)
        indices = aux_list.argsort()
        new_twoD = self.array.take(indices, axis=0)
        
        kw = self._get_persistent_attrs()
        kw.update(kwargs)
        return Table(header = self.Header, rows = new_twoD, **kw)
    
    def getColumns(self, columns, **kwargs):
        """Return a slice of columns"""
        # check whether we have integer columns
        
        if isinstance(columns, str):
            columns = [columns]
        
        is_int = min([isinstance(val, int) for val in columns])
        indexes = []
        if is_int:
            indexes = columns
        else:
            indexes = [self.Header.index(head) for head in columns]
        
        if self._row_ids:
            # we disallow reordering of identifiers, and ensure they are only
            # presented once
            for val in range(self._row_ids):
                try:
                    indexes.remove(val)
                except ValueError:
                    pass
            indexes = range(self._row_ids) + indexes
        
        columns = numpy.take(numpy.asarray(self.Header, dtype="O"),
                               indexes)
        new = numpy.take(self.array, indexes, axis=1)
        
        kw = self._get_persistent_attrs()
        kw.update(kwargs)
        return Table(header = columns, rows = new, **kw)
    
    def getDisjointRows(self, rows, **kwargs):
        """Return the nominated disjoint rows."""
        if isinstance(rows, str):
            rows = [rows]
        
        indexes = []
        for row in rows:
            idx, drop = self.template.interpretIndex(row)
            indexes.append(idx[0])
        
        new = self.array.take(indexes, axis=0)
        
        kw = self._get_persistent_attrs()
        kw.update(kwargs)
        return Table(header = self.Header, rows = new, **kw)
    
    def withNewColumn(self, new_column, callback, columns = None, **kwargs):
        """Returns a new table with an additional column, computed using
        callback.
        
        Arguments:
            - new_column: new column heading
            - columns: the columns whose values determine whether a row is to
            be included.
            - callback: Can be a function, which takes the sub-row delimited
            by columns and returns True/False, or a string representing valid
            python code to be evaluated."""
        
        if isinstance(columns, str):
            columns = (columns,)
        
        if columns is not None:
            num_columns = len(columns)
        else:
            num_columns = None
        
        if not callable(callback):
            data = self
            cols = columns
        else:
            data = self.array
            cols = map(self.Header.index, columns)
        
        twoD = [list(row) + [self._callback(callback, row, cols,
                num_columns)] for row in data]
        
        kw = self._get_persistent_attrs()
        kw.update(kwargs)
        return Table(header = self.Header + [new_column], rows = twoD, **kw)
    
    def getDistinctValues(self, column):
        """returns the set of distinct values for the named column(s)"""
        columns = [column, [column]][type(column) == str]
        data = self.getRawData(column)
        
        if len(columns) > 1:
            data = [tuple(row) for row in data]
        
        return set(data)
    
    def joined(self, other_table, columns_self=None, columns_other=None,
                inner_join=True, **kwargs):
        """returns a new table containing the join of this table and
        other_table. Default behaviour is the natural inner join. Checks for
        equality in the specified columns (if provided) or all columns; a
        combined row is included in the output if all indices match exactly. A
        combined row contains first the row of this table, and then columns
        from the other_table that are not key columns (i.e. not specified in
        columns_other). The order (of self, then other)
        is preserved. The column headers of the output are made unique by
        replacing the headers of other_table with
        <other_table.Title>_<other_table.Header>.
        
        Arguments:
            - other_table: A table object which will be joined with this
              table. other_table must have a title.
            - columns_self, columns_other: indices of key columns that will
              be compared in the join operation. Can be either column index,
              or a string matching the column header. The order matters, and
              the dimensions of columns_self and columns_other have to match.
              A row will be included in the output iff
              self[row][columns_self[i]]==other_table[row][columns_other[i]]
              for all i
            - inner_join: if False, the outer join of the two tables is
              returned.
        """
        
        if other_table.Title is None:
            raise RuntimeError, "Cannot join if a other_table.Title is None"
        elif self.Title == other_table.Title:
            raise RuntimeError, "Cannot join if a table.Title's are equal"
        
        columns_self = [columns_self,[columns_self]][type(columns_self)==str]
        columns_other = [columns_other,
                            [columns_other]][type(columns_other)==str]
        if not inner_join:
            assert columns_self is None and columns_other is None, "Cannot "\
                                "specify column indices for an outer join"
            columns_self = []
            columns_other = []
        
        if columns_self is None and columns_other is None:
            # we do the natural inner join
            columns_self=[]
            columns_other=[]
            for col_head in self.Header:
                if col_head in other_table.Header:
                    columns_self.append(self.Header.index(col_head))
                    columns_other.append(other_table.Header.index(col_head))
        elif columns_self is None or columns_other is None:
            # the same column labels will be used for both tables
            columns_self = columns_self or columns_other
            columns_other = columns_self or columns_other
        elif len(columns_self)!=len(columns_other):
            raise RuntimeError("Error during table join: key columns have "\
                  "different dimensions!")
        
        # create new 2d list for the output
        joined_table=[]
        
        #resolve column indices from Header, if necessary
        columns_self_indices=[]
        columns_other_indices=[]
        for col in columns_self:
            if type(col)==int:
                columns_self_indices.append(col)
            else:
                columns_self_indices.append(self.Header.index(col))
        
        for col in columns_other:
            if type(col)==int:
                columns_other_indices.append(col)
            else:
                columns_other_indices.append(other_table.Header.index(col))
        # create a mask of which columns of the other_table will end up in the
        # output
        output_mask_other=[]
        for col in range(0,len(other_table.Header)):
            if not (col in columns_other_indices):
                output_mask_other.append(col)
        # use a dictionary for the key lookup
        # key dictionary for other_table.
        # key is a tuple made from specified columns; data is the row index
        # for lookup...
        key_lookup={}
        row_index=0
        for row in other_table:
            #insert new entry for each row
            key=tuple([row[col] for col in columns_other_indices])
            if key in key_lookup:
                key_lookup[key].append(row_index)
            else:
                key_lookup[key]=[row_index]
            row_index=row_index+1
            
        for this_row in self:
            # assemble key for query of other_table
            key=tuple([this_row[col] for col in columns_self_indices])
            if key in key_lookup:
                for output_row_index in key_lookup[key]:
                    other_row=[other_table[output_row_index,c] \
                                                  for c in output_mask_other]
                    joined_table.append(list(this_row) + other_row)
        
        new_header=self.Header+[other_table.Title+"_"+other_table.Header[c] \
                                                  for c in output_mask_other]
        if not joined_table:
            # YUK, this is to stop dimension check in DictArray causing
            # failures
            joined_table = numpy.empty((0,len(new_header)))
        
        return Table(header=new_header, rows=joined_table, **kwargs)
    
    def summed(self, indices=None, col_sum=True, strict=True, **kwargs):
        """returns the sum of numerical values for column(s)/row(s)
        
        Arguments:
            - indices: column name(s) or indices or row indices
            - col_sum: sums values in the indicated column, the default. If
              False, returns the row sum.
            - strict: if False, ignores cells with non-numeric data in the
              column/row."""
        
        all = indices is None
        
        if type(indices) == str:
            assert col_sum, "Must use row integer indices"
            indices = self.Header.index(indices)
        elif type(indices) == int: # a single index
            indices = [indices]
        elif not all:
            raise RuntimeError("unknown indices type: %s" % indices)
        
        if not all:
            vals = self.array.take([indices], axis=[0,1][col_sum]).flatten()
            if strict:
                result = vals.sum()
            else:
                result = sum(v for v in vals if type(v)!=str)
        else:
            # a multi-rowed result
            if col_sum:
                vals = self.array
            else:
                vals = self.array.transpose() 
            
            if strict:
                result = vals.sum(axis=0).tolist()
            else:
                result = []
                append = result.append
                # we need to iterate over the elements to be summed, so we
                # have to transpose
                for row in vals.transpose():
                    try:
                        num = row.sum()
                    except TypeError:
                        num = sum(r for r in row if type(r) != str)
                    append(num)
            
        
        return result
    
    def normalized(self, by_row=True, denominator_func=None, **kwargs):
        """returns a table with elements expressed as a fraction according
        to the results from func
        
        Arguments:
            - by_row: normalisation done by row
            - denominator_func: a callback function that takes an array and
              returns a value to be used as the denominator. Default is sum."""
        
        if denominator_func:
            data = self.array
            if not by_row:
                data = data.transpose()
            denominators = [denominator_func(row) for row in data]
        else:
            denominators = self.summed(col_sum=not by_row)
        
        if by_row:
            values = self.array
        else:
            values = self.array.transpose()
        
        rows = [values[i]/denom for i, denom in enumerate(denominators)]
        rows = numpy.array(rows)
        
        if not by_row:
            rows = rows.transpose()
        
        return Table(header=self.Header, rows=rows, **kwargs)
    
    def transposed(self, new_column_name, select_as_header=None, **kwargs):
        """returns the transposed table.
        
        Arguments:
            - new_column_name: the existing header will become a column with
              this name
            - select_as_header: current column name containing data to be used
              as the header. Defaults to the first column.
        """
        select_as_header = select_as_header or self.Header[0]
        assert select_as_header in self.Header, \
                    '"%s" not in table Header' % select_as_header
        
        raw_data = self.getRawData()
        raw_data.insert(0, self.Header)
        transposed = numpy.array(raw_data, dtype='O')
        transposed = transposed.transpose()
        
        # indices for the header and non header rows
        header_index = self.Header.index(select_as_header)
        
        data_indices = range(0, header_index)+range(header_index+1,
                                                    len(transposed))
        
        header = list(numpy.take(transposed, [header_index], axis=0)[0])
        header = [new_column_name]+header[1:] # [1:] slice excludes old name
        rows = numpy.take(transposed, data_indices, axis=0)
        return Table(header=header, rows=rows, **kwargs)
    



#!/usr/bin/env python
"""Tests of BLAST parser.
"""
from string import split, strip
from cogent.util.unit_test import TestCase, main
from cogent.parse.blast import iter_finder, query_finder, iteration_set_finder,\
    is_blast_junk, is_blat_junk, make_label, PsiBlastQueryFinder, \
    TableToValues, \
    PsiBlastTableParser, PsiBlastFinder, GenericBlastParser9, \
    PsiBlastParser9, LastProteinIds9, QMEBlast9, QMEPsiBlast9, \
    fastacmd_taxonomy_splitter, FastacmdTaxonomyParser

__author__ = "Micah Hamady"
__copyright__ = "Copyright 2007-2016, The Cogent Project"
__credits__ = ["Micah Hamady", "Rob Knight"]
__license__ = "GPL"
__version__ = "1.9"
__maintainer__ = "Micah Hamady"
__email__ = "hamady@colorado.edu"
__status__ = "Production"

class BlastTests(TestCase):
    """Tests of top-level functions"""

    def setUp(self):
        """Define some standard data"""
        self.rec = """# BLASTP 2.2.10 [Oct-19-2004]
# Iteration: 1
# Query: ece:Z4181
# Database: db/everything.faa
# Fields: Query id, Subject id, % identity, alignment length, mismatches, gap openings, q. start, q. end, s. start, s. end, e-value, bit score
ece:Z4181	ece:Z4181	100.00	110	0	0	1	110	1	110	3e-47	 187
ece:Z4181	ecs:ECs3717	100.00	110	0	0	1	110	1	110	3e-47	 187
ece:Z4181	cvi:CV2421	41.67	72	42	0	39	110	29	100	2e-06	52.8
# BLASTP 2.2.10 [Oct-19-2004]
# Iteration: 2
# Query: ece:Z4181
# Database: db/everything.faa
# Fields: Query id, Subject id, % identity, alignment length, mismatches, gap openings, q. start, q. end, s. start, s. end, e-value, bit score
ece:Z4181	ece:Z4181	100.00	110	0	0	1	110	1	110	3e-54	 211
ece:Z4181	ecs:ECs3717	100.00	110	0	0	1	110	1	110	3e-54	 211
ece:Z4181	cvi:CV2421	41.67	72	42	0	39	110	29	100	2e-08	59.0
ece:Z4181	sfl:CP0138	33.98	103	57	2	8	110	6	97	6e-06	50.5
ece:Z4181	spt:SPA2730	37.50	72	45	0	39	110	30	101	1e-05	49.8
ece:Z4181	sec:SC2804	37.50	72	45	0	39	110	30	101	1e-05	49.8
ece:Z4181	stm:STM2872	37.50	72	45	0	39	110	30	101	1e-05	49.8""".split('\n')

        self.rec2 = """# BLASTP 2.2.10 [Oct-19-2004]
# Iteration: 1
# Query: ece:Z4181
# Database: db/everything.faa
# Fields: Query id, Subject id, % identity, alignment length, mismatches, gap openings, q. start, q. end, s. start, s. end, e-value, bit score
ece:Z4181	ece:Z4181	100.00	110	0	0	1	110	1	110	3e-47	 187
ece:Z4181	ecs:ECs3717	100.00	110	0	0	1	110	1	110	3e-47	 187
ece:Z4181	cvi:CV2421	41.67	72	42	0	39	110	29	100	2e-06	52.8
# BLASTP 2.2.10 [Oct-19-2004]
# Iteration: 2
# Query: ece:Z4181
# Database: db/everything.faa
# Fields: Query id, Subject id, % identity, alignment length, mismatches, gap openings, q. start, q. end, s. start, s. end, e-value, bit score
ece:Z4181	ece:Z4181	100.00	110	0	0	1	110	1	110	3e-54	 211
ece:Z4181	ecs:ECs3717	100.00	110	0	0	1	110	1	110	3e-54	 211
ece:Z4181	cvi:CV2421	41.67	72	42	0	39	110	29	100	2e-08	59.0
ece:Z4181	sfl:CP0138	33.98	103	57	2	8	110	6	97	6e-06	50.5
ece:Z4181	spt:SPA2730	37.50	72	45	0	39	110	30	101	1e-05	49.8
ece:Z4181	sec:SC2804	37.50	72	45	0	39	110	30	101	1e-05	49.8
ece:Z4181	stm:STM2872	37.50	72	45	0	39	110	30	101	1e-05	49.8
# BLASTP 2.2.10 [Oct-19-2004]
# Iteration: 1
# Query: ece:Z4182
# Database: db/everything.faa
# Fields: Query id, Subject id, % identity, alignment length, mismatches, gap openings, q. start, q. end, s. start, s. end, e-value, bit score
ece:Z4182	ece:Z4182	100.00	110	0	0	1	110	1	110	3e-47	 187
ece:Z4182	ecs:ECs3718	100.00	110	0	0	1	110	1	110	3e-47	 187
ece:Z4182	cvi:CV2422	41.67	72	42	0	39	110	29	100	2e-06	52.8""".split('\n')

        self.rec3 = """# BLASTP 2.2.10 [Oct-19-2004]
# Iteration: 1
# Query: ece:Z4181
# Database: db/everything.faa
# Fields: Query id, Subject id, % identity, alignment length, mismatches, gap openings, q. start, q. end, s. start, s. end, e-value, bit score
ece:Z4181	ece:Z4181	100.00	110	0	0	1	110	1	110	3e-47	 187
ece:Z4181	ecs:ECs3717	100.00	110	0	0	1	110	1	110	3e-47	 187
ece:Z4181	spt:SPA2730	37.50	72	45	0	39	110	30	101	1e-05	49.8
# BLASTP 2.2.10 [Oct-19-2004]
# Iteration: 2
# Query: ece:Z4181
# Database: db/everything.faa
# Fields: Query id, Subject id, % identity, alignment length, mismatches, gap openings, q. start, q. end, s. start, s. end, e-value, bit score
ece:Z4181	ecs:ECs3717	100.00	110	0	0	1	110	1	110	3e-54	 211
ece:Z4181	cvi:CV2421	41.67	72	42	0	39	110	29	100	2e-08	59.0
# BLASTP 2.2.10 [Oct-19-2004]
# Iteration: 1
# Query: ece:Z4182
# Database: db/everything.faa
# Fields: Query id, Subject id, % identity, alignment length, mismatches, gap openings, q. start, q. end, s. start, s. end, e-value, bit score
ece:Z4182	ece:Z4182	100.00	110	0	0	1	110	1	110	3e-47	 187
ece:Z4182	cvi:CV2422	41.67	72	42	0	39	110	29	100	2e-06	52.8""".split('\n')


    def test_iter_finder(self):
        """iter_finder should split on lines starting with '# Iteration:'"""
        lines = 'abc\n# Iteration: 3\ndef'.split('\n')
        self.assertEqual(map(iter_finder,lines), [False, True, False])

    def test_query_finder(self):
        """query_finder should split on lines starting with '# Query:'"""
        lines = 'abc\n# Query: dfdsffsd\ndef'.split('\n')
        self.assertEqual(map(query_finder,lines), [False, True, False])
    
    def test_iteration_set_finder(self):
        """iter_finder should split on lines starting with '# Iteration:'"""
        lines = 'abc\n# Iteration: 3\ndef\n# Iteration: 1'.split('\n')
        self.assertEqual(map(iteration_set_finder,lines), \
            [False, False, False, True])

    def test_is_junk(self):
        """is_junk should reject an assortment of invalid lines"""
        #Note: testing two functions that call it instead of function itself
        lines = 'abc\n# BLAST blah blah\n   \n# BLAT blah\n123'.split('\n')
        self.assertEqual(map(is_blast_junk, lines), \
            [False, True, True, False, False])
        self.assertEqual(map(is_blat_junk, lines), \
            [False, False, True, True, False])

    def test_make_label(self):
        """make_label should turn comment lines into (key, val) pairs"""
        a = 'this test will fail: no # at start'
        b = '#this test will fail because no colon'
        c = '# Iteration: 1'
        d = '# Query: ece:Z4147  ygdP; putative invasion protein [EC:3.6.1.-]'
        e =  '#Iteration: 1'    #no space after the hash
        self.assertRaises(ValueError, make_label, a)
        self.assertRaises(ValueError, make_label, b)
        #Note that we _do_ map the data type of known values value, so the
        #value of the iteration will be 1, not '1'
        self.assertEqual(make_label(c), ('ITERATION', 1))
        self.assertEqual(make_label(d), ('QUERY', \
            'ece:Z4147  ygdP; putative invasion protein [EC:3.6.1.-]'))
        self.assertEqual(make_label(e), ('ITERATION', 1))

    def test_TableToValues(self):
        """TableToValues should convert itself into the correct type."""
        constructors = {'a':int, 'b':float, 'c':str}
        table=[['c','b','a','d'], ['1.5', '3.5', '2', '2.5'],['1','2','3','4']]
        self.assertEqual(TableToValues(table, constructors), \
            ([['1.5',3.5,2,'2.5'],['1',2.0,3,'4']], ['c','b','a','d']))
        #check that it works with supplied header
        self.assertEqual(TableToValues(table[1:], constructors, list('cbad')), \
            ([['1.5',3.5,2,'2.5'],['1',2.0,3,'4']], ['c','b','a','d']))

    def test_PsiBlastTableParser(self):
        """PsiBlastTableParser should wrap values in table."""
        fields = map(strip,
'Query id, Subject id, % identity, alignment length, mismatches, gap openings, q. start, q. end, s. start, s. end, e-value, bit score'.split(','))
        
        table = map(split, """ece:Z4147       ece:Z4147       100.00  176     0       0       1       176     1       176     2e-89    328
        ece:Z4147       ecs:ECs3687     100.00  176     0       0       1       176     1       176     2e-89    328
        ece:Z4147       ecc:c3425       100.00  176     0       0       1       176     1       176     2e-89    328
        ece:Z4147       sfl:SF2840      100.00  176     0       0       1       176     1       176     2e-89    328""".split('\n'))
        headed_table = [fields] + table
        new_table, new_fields = PsiBlastTableParser(headed_table)
        self.assertEqual(new_fields, fields)
        self.assertEqual(len(new_table), 4)
        self.assertEqual(new_table[1], ['ece:Z4147', 'ecs:ECs3687', 100.0, \
            176, 0, 0, 1, 176, 1, 176, 2e-89, 328])
        
    def test_GenericBlastParser9(self):
        """GenericBlastParser9 should read blast's tabular format (#9)."""
        rec = self.rec
        p = GenericBlastParser9(rec, PsiBlastFinder)
        result = list(p)
        self.assertEqual(len(result), 2)
        first, second = result
        self.assertEqual(first[0], {'ITERATION':1,'QUERY':'ece:Z4181',\
            'DATABASE':'db/everything.faa', 'FIELDS':'Query id, Subject id, % identity, alignment length, mismatches, gap openings, q. start, q. end, s. start, s. end, e-value, bit score'})
        self.assertEqual(len(first[1]), 3)
        self.assertEqual(second[0]['ITERATION'], 2)
        self.assertEqual(len(second[1]), 7)
        self.assertEqual(second[1][-1], \
'ece:Z4181   stm:STM2872 37.50   72  45  0   39  110 30  101 1e-05   49.8'.split())

    def test_PsiBlastParser9(self):
        """PsiBlastParser9 should provide convenient results for format #9."""
        result = PsiBlastParser9(self.rec2)
        self.assertEqual(len(result), 2)
        assert 'ece:Z4181' in result
        assert 'ece:Z4182' in result
        first = result['ece:Z4181']
        second = result['ece:Z4182']
        self.assertEqual(len(first), 2)
        self.assertEqual(len(second), 1)
        iter_1 = first[0]
        iter_2 = first[1]
        self.assertEqual(len(iter_1), 3)
        self.assertEqual(len(iter_2), 7)
        iter_1_2 = second[0]
        self.assertEqual(len(iter_1_2), 3)
        self.assertEqual(len(result['ece:Z4181'][1][3]), 12)
        self.assertEqual(result['ece:Z4181'][1][3]['ALIGNMENT LENGTH'], 103)


    def test_LastProteinIds9(self):
        """LastProteinIds9 should give last protein ids in iter"""
        result = LastProteinIds9(self.rec)
        self.assertEqual(result, ['ece:Z4181', 'ecs:ECs3717', 'cvi:CV2421',\
            'sfl:CP0138', 'spt:SPA2730', 'sec:SC2804', 'stm:STM2872'])
        #should also work if threshold set
        result = LastProteinIds9(self.rec, False, threshold=8e-6)
        self.assertEqual(result, ['ece:Z4181', 'ecs:ECs3717', 'cvi:CV2421',\
            'sfl:CP0138'])
        #should work on multiple records
        result = map(LastProteinIds9, PsiBlastQueryFinder(self.rec2))
        self.assertEqual(len(result), 2)
        self.assertEqual(result[0], ['ece:Z4181', 'ecs:ECs3717', 'cvi:CV2421',\
            'sfl:CP0138', 'spt:SPA2730', 'sec:SC2804', 'stm:STM2872'])
        self.assertEqual(result[1], ['ece:Z4182','ecs:ECs3718','cvi:CV2422'])

    def test_QMEBlast9(self):
        """QMEBlast9 should return expected lines from all iterations"""
        self.assertFloatEqual(QMEBlast9(self.rec3), [\
                ('ece:Z4181','ece:Z4181',3e-47),
                ('ece:Z4181','ecs:ECs3717',3e-47),
                ('ece:Z4181','spt:SPA2730', 1e-5),
                ('ece:Z4181','ecs:ECs3717',3e-54),  #WARNING: allows duplicates
                ('ece:Z4181','cvi:CV2421',2e-8),
                ('ece:Z4182','ece:Z4182',3e-47),
                ('ece:Z4182','cvi:CV2422',2e-6),
                ])

    def test_QMEPsiBlast9(self):
        """QMEPsiBlast9 should only return items from last iterations"""
        self.assertFloatEqual(QMEPsiBlast9(self.rec3), [\
                ('ece:Z4181','ecs:ECs3717',3e-54),
                ('ece:Z4181','cvi:CV2421',2e-8),
                ('ece:Z4182','ece:Z4182',3e-47),
                ('ece:Z4182','cvi:CV2422',2e-6),
                ])

    def test_fastacmd_taxonomy_splitter(self):
        """fastacmd_taxonomy_splitter should split records into groups"""
        text = """NCBI sequence id: gi|3021565|emb|AJ223314.1|PSAJ3314
NCBI taxonomy id: 3349
Common name: Scots pine
Scientific name: Pinus sylvestris

NCBI sequence id: gi|37777029|dbj|AB108787.1|
NCBI taxonomy id: 228610
Common name: cf. Acremonium sp. KR21-2
Scientific name: cf. Acremonium sp. KR21-2

""".splitlines()
        recs = list(fastacmd_taxonomy_splitter(text))
        self.assertEqual(len(recs), 2)
        self.assertEqual(recs[0], text[:5]) #includes trailing blank

    def test_FastaCmdTaxonomyParser(self):
        """FastaCmdTaxonomyParser should parse taxonomy record to dict"""
        text = """NCBI sequence id: gi|3021565|emb|AJ223314.1|PSAJ3314
NCBI taxonomy id: 3349
Common name: Scots pine
Scientific name: Pinus sylvestris

NCBI sequence id: gi|37777029|dbj|AB108787.1|
NCBI taxonomy id: 228610
Common name: cf. Acremonium sp. KR21-2
Scientific name: cf. Acremonium sp. KR21-2

""".splitlines()
        recs = list(FastacmdTaxonomyParser(text))
        self.assertEqual(len(recs), 2)
        for r in recs:
            self.assertEqual(sorted(r.keys()), ['common_name','scientific_name',
                'seq_id', 'tax_id'])
        r0, r1 = recs
        self.assertEqual(r0['tax_id'], '3349')
        self.assertEqual(r0['common_name'], 'Scots pine')
        self.assertEqual(r0['scientific_name'], 'Pinus sylvestris')
        self.assertEqual(r0['seq_id'], 'gi|3021565|emb|AJ223314.1|PSAJ3314')
        self.assertEqual(r1['tax_id'], '228610')

        
                            
if __name__ == "__main__":
    main()



#!/usr/bin/env python
"""microarray_normalize.py: provides functions to normalize array data.

Will use for testing normalization methods against simulated data.

Implementation notes:

All normalization methods should use the interface f(a) -> n, where a is an 
array in which the rows are genes (or probes) and the columns are samples 
(i.e. chips); n is an array of the same shape that contains the normalized 
values.

All probe consolidation methods should be f(probes, groups) -> genes where 
probes is the array of probes (rows = probes, cols = samples), and genes
is the array of genes.

All platform consolidation methods should be f([p_1, p_2, ...]) -> g where
the input is a list of probe arrays for each platform, and the output is a
single gene array.

All missing value imputation methods should be f(a, m) -> n where a is an array 
where each row is a gene (or probe), m is a dict such that m[(x,y)] is True if
a[x,y] is missing (we are assuming that missing values are rare), and n is a
dense array containing imputed values at missing positions.

Revision History
10/28/05 Rob Knight: file created.
11/10/05 Micah Hamady: merged w/my implementations
"""
from cogent.maths.stats.distribution import ndtri
from numpy import ceil, arange, argsort, sort, array, log2, zeros, ravel, \
                  transpose, take, mean, std
from numpy.linalg import svd

__author__ = "Rob Knight"
__copyright__ = "Copyright 2007-2016, The Cogent Project"
__credits__ = ["Rob Knight", "Micah Hamady", "Daniel McDonald"]
__license__ = "GPL"
__version__ = "1.9"
__maintainer__ = "Rob Knight"
__email__ = "rob@spot.colorado.edu"
__status__ = "Development"

def zscores(a):
    """Converts a to zscores in each col, i.e. subtract mean and div by stdev."""
    return (a - mean(a,axis=0)) / std(a, axis=0)

def logzscores(a):
    """Takes log (base 2) of values then computes zscores"""
    return zscores(log2(a))

def ranks(a):
    """Converts a to absolute ranks in each col, 0 = smallest value.
    
    Doesn't break ties: instead, assigns arbitrary ranks.
    """
    return(argsort(argsort(a,0),0))

def quantiles(a):
    """Converts a to quantiles p(x<a) in each col, 0=smallest, (n-1)/n=largest.

    Doesn't break ties.
    """
    return ranks(a)/float(len(a))

def make_quantile_normalizer(dist):
    """Returns f(a) that converts to the quantile value in each col.

    dist should be an array with bins equally spaced from 0 to 1, giving
    the value in each bin (i.e. cumulative prob of f(x) at f(i/len(dist))
    should be stored in dist[i]) -- can generate from distribution or generate
    empirically.
    """
    def qn(a):
        result = (quantiles(a)*len(dist)).astype('i')
        return take(dist, result)
    return qn

def make_normal_quantile_normalizer(mean, sd, bins=1000):
    """returns f(a) that converts a to the specified normal distribution."""
    dist = array([ndtri(i)*sd for i in arange(1.0/bins,1,1.0/bins)])
    dist = (dist * sd) + mean
    return make_quantile_normalizer(dist)

def make_empirical_quantile_normalizer(data):
    """returns f(a) that converts a to the same distribution as data."""
    return make_quantile_normalizer(sort(data))

def geometric_mean(vals):
    """
    Compute geometric mean

    vals: array of values to compute geometric mean of
    """
    if len(vals) < 2:
        raise ValueError, "Not enought values for geometric mean."
    gmean = 1.0 
    root = 1.0 / len(vals)

    return reduce(lambda x, y: x * y, pow(vals, root))

#FIXME: no formal unit tests for remaining functions.

def svd_cols(a):
    """Returns svd by cols. Note: usually want to discard col[0] of result.
    
    WARNING: will fail on redundant rows! Need to filter out redundant rows
    beforehand, then put back in after normalization. Picks the matrix that
    has the same shape as a.
    """
    u, v, w = svd(a)
    if u.shape == a.shape:
        return u
    elif w.shape == a.shape:
        return w
    else:
        raise TypeError, "Neither u nor w same shape as a."

def rank_rows_by_variance(a):
    """Returns array containing indices 0..num_rows, ranked by variance."""
    return argsort(std(a,1))

def n_least_quantile_var(a, n):
    """Finds the best n rows in a with minimum quantile variance."""
    return rank_row_by_variance(quantiles(a))[:n]

def make_constant_rank_normalizer(constant_rank_indices=None):
    """Returns normalizer that divs by constant_rank_indices.
    
    If constant_rank_indices is an int rather than a list, picks best n.
    """
    def constant_rank_normalizer(a):
        if isinstance(int, constant_rank_indices):
            constant_rank_indices = rank_rows_by_variance(a,\
                constant_rank_indices)
        h = take(a, constant_rank_indices)
        return a / mean(h)
    return constant_rank_normalizer

def omit_rows(a, f):
    """Returns copy of a where f(a[i]) == True, and map of [new]->[old] index.
    
    All omit_rows functions will return both these values.
    """
    mask = array(map(f, a))
    coords = nonzero(mask)
    return take(a, coords), coords

def omit_rows_below_mean_threshold(a, t):
    """Returns copy of a without rows that have mean below threshold."""
    return omit_rows(a, lambda f: mean(f) >= t)

def omit_rows_below_max_threshold(a, t):
    """Returns copy of a without rows that have max val below threshold."""
    return omit_rows(a, lambda f: max(f) >= t)

def group_rows(a, groups):
    """Converts a into list of groups, specified as lists of lists of indices.

    i.e. groups should be a list of groups, where each group contains the 
    indices of the rows that belong to it.
    """
    return [take(a, indices) for indices in groups]

def max_per_group(a, groups):
    """Returns array with max item for each col for each group."""
    return array(map(max, group_rows(a, groups)))

def mean_per_group(a, groups):
    """Returns array with mean for each col for each group."""
    return array(map(mean, group_rows(a, groups)))

def housekeeping_gene_normalize(a, housekeeping_gene_indexes):
    """
    Normalize matrix based on mean/std dev of housekeeping genes 

    Need to refactor to use more effiecient array operations (in place)
    
    a: microarray data. expects rows to be genes, columns to be samples 
    housekeeping_gene_indexes: list of indexes of genes (rows) that represent
        the housekeeping set
    """
    norm_values = ravel(take(a, housekeeping_gene_indexes, 1))
    hk_mean = mean(norm_values)
    hk_std_dev = std(norm_values)
 
    if not hk_std_dev:
        raise NormalizationError, "Cannot normalize, std dev is zero." 
    
    return (a - hk_mean) / hk_std_dev





#!/usr/bin/env python
# test_sffinfo.py

import os
import shutil
import tempfile
from cogent.util.unit_test import TestCase, main
from cogent.app.util import ApplicationError
from cogent.app.sffinfo import (
    ManyValuedParameter, Sffinfo, sffinfo_from_file)

__author__ = "Kyle Bittinger"
__copyright__ = "Copyright 2007-2016, The Cogent Project"
__credits__ = ["Kyle Bittinger"]
__license__ = "GPL"
__version__ = "1.9"
__maintainer__ = "Kyle Bittinger"
__email__ = "kylebittinger@gmail.com"
__status__ = "Prototype"


class ManyValuedParameterTests(TestCase):
    def test_init(self):
        """__init__() should set appropriate class variables."""
        p = ManyValuedParameter(None, None)
        self.assertEqual(p.Name, None)
        self.assertEqual(p.Value, None)
        self.assertEqual(p.Delimiter, None)
        self.assertEqual(p.Quote, None)

        p = ManyValuedParameter('-', 'a', Values=['abc'])
        self.assertEqual(p.Value, ['abc'])

    def test_append(self):
        """append() should append values to Value class attribute."""
        p = ManyValuedParameter(None, None)
        p.append('abc')
        p.append(3)
        self.assertEqual(p.Value, ['abc', 3])

    def test_on(self):
        """on() should alias append()."""
        p = ManyValuedParameter(None, None)
        p.on('abc')
        p.on(3)
        self.assertEqual(p.Value, ['abc', 3])

    def test_str(self):
        """__str__() should produce quoted, delimited string of parameter values."""
        p = ManyValuedParameter(None, None)
        p.append('abc')
        p.append(3)
        self.assertEqual(str(p), 'abc3')

        p = ManyValuedParameter(None, None, Quote='"', ValueDelimiter=',')
        p.append('abc')
        p.append(3)
        self.assertEqual(str(p), '"abc","3"')


class SffinfoTests(TestCase):
    def setUp(self):
        test_dir = os.path.dirname(os.path.dirname(os.path.abspath(__file__)))
        self.sff_fp = os.path.join(test_dir, 'data', 'test.sff')

    def test_base_command(self):
        """BaseCommand should include non-positional parameters."""
        s = Sffinfo()
        expected = 'cd "%s/"; sffinfo' % os.getcwd()
        self.assertEqual(s.BaseCommand, expected)

        s.Parameters['-a'].on()
        expected = 'cd "%s/"; sffinfo -a' % os.getcwd()
        self.assertEqual(s.BaseCommand, expected)

        # accession number parameters should not be included in base command
        s.Parameters['-a'].off()
        s.Parameters['accno'].on('12345ABC')
        expected = 'cd "%s/"; sffinfo' % os.getcwd()
        self.assertEqual(s.BaseCommand, expected)

    def test_changing_working_dir(self):
        """WorkingDir should be created and included in BaseCommand."""
        # Directory is not created, only filename is returned
        working_dir = tempfile.mktemp()
        self.assertRaises(OSError, os.rmdir, working_dir)
        
        a = Sffinfo(WorkingDir=working_dir)
        expected = 'cd "%s/"; sffinfo' % working_dir
        self.assertEqual(a.BaseCommand, expected)

        # Removing the directory is proof that it was created.  If the
        # directory is not there, an OSError will be raised.
        os.rmdir(working_dir)

    def test_input_handler(self):
        """Sffinfo should decorate input handler output with accession numbers"""
        my_accno = '12345ABC'
        a = Sffinfo()
        a.Parameters['accno'].on(my_accno)
        self.assertEqual(a.InputHandler, '_input_handler_decorator')
        observed = getattr(a, a.InputHandler)(self.sff_fp)
        expected = '"%s" %s' % (self.sff_fp, my_accno)
        self.assertEqual(observed, expected)

    def test_call(self):
        """Simple sffinfo call should produce expected output."""
        a = Sffinfo()
        app_results = a(self.sff_fp)
        observed = app_results['StdOut'].read()
        self.assertEqual(observed, sffinfo_output)

    def test_call_with_accno(self):
        """Sffinfo accession number parameters should filter output."""
        # Valid accession number
        a = Sffinfo()
        a.Parameters['accno'].on('FA6P1OK01CGMHQ')
        app_results = a(self.sff_fp)
        observed = app_results['StdOut'].read()
        self.assertEqual(observed, sffinfo_output)

        # Invalid accession number
        a = Sffinfo()
        a.Parameters['accno'].on('AAAAAAAAAAAAAA')
        app_results = a(self.sff_fp)
        observed = app_results['StdOut'].read()
        self.assertEqual(observed, empty_sffinfo_output)

    def test_call_with_flags(self):
        """Sffinfo flags should alter output as expected."""
        # -a flag
        a = Sffinfo()
        a.Parameters['-a'].on()
        app_results = a(self.sff_fp)
        observed = app_results['StdOut'].read()
        self.assertEqual(observed, 'FA6P1OK01CGMHQ\n')

        # -s flag
        a = Sffinfo()
        a.Parameters['-s'].on()
        app_results = a(self.sff_fp)
        observed = app_results['StdOut'].read()
        expected = (
            '>FA6P1OK01CGMHQ length=48 xy=0892_1356 region=1 '
            'run=R_2008_05_28_17_11_38_\n'
            'ATCTGAGCTGGGTCATAGCTGCCTCCGTAGGAGGTGCCTCCCTACGGC\n'
            )
        self.assertEqual(observed, expected)

        # -q flag
        a = Sffinfo()
        a.Parameters['-q'].on()
        app_results = a(self.sff_fp)
        observed = app_results['StdOut'].read()
        expected = (
            '>FA6P1OK01CGMHQ length=48 xy=0892_1356 region=1 '
            'run=R_2008_05_28_17_11_38_\n'
            '32 32 32 32 32 32 32 25 25 21 21 21 28 32 32 31 30 30 32 32 32 '
            '33 31 25 18 18 20 18 32 30 28 23 22 22 24 28 18 19 18 16 16 16 '
            '17 18 13 17 27 21\n')
        self.assertEqual(observed, expected)

        # -f flag
        a = Sffinfo()
        a.Parameters['-f'].on()
        app_results = a(self.sff_fp)
        observed = app_results['StdOut'].read()
        expected = (
            '>FA6P1OK01CGMHQ xy=0892_1356 region=1 run=R_2008_05_28_17_11_38_\n'
            'A,1.02 C,0.00 G,0.00 T,0.99 A,0.00 C,1.00 G,0.00 T,1.00 A,0.00 C,0.00\n'
            'G,1.00 T,0.00 A,1.10 C,0.00 G,1.08 T,0.00 A,0.00 C,1.46 G,0.00 T,0.88\n'
            'A,0.18 C,0.00 G,2.69 T,1.01 A,0.08 C,0.96 G,0.00 T,0.02 A,0.92 C,0.08\n'
            'G,0.00 T,0.98 A,0.68 C,0.00 G,0.89 T,0.00 A,0.00 C,1.15 G,0.00 T,1.13\n'
            'A,0.00 C,0.02 G,1.12 T,0.05 A,0.15 C,1.84 G,0.00 T,1.10 A,0.00 C,2.47\n'
            'G,0.96 T,0.86 A,1.06 C,0.00 G,1.96 T,0.12 A,0.93 C,0.13 G,1.65 T,1.06\n'
            'A,0.06 C,0.00 G,0.99 T,0.00 A,0.00 C,1.87 G,0.44 T,1.08 A,0.00 C,3.25\n'
            'G,0.09 T,0.97 A,0.50 C,1.00 G,1.72 T,0.07 A,0.00 C,0.92 \n')
        self.assertEqual(observed, expected)


class SffinfoFunctionTests(TestCase):
    def test_sffinfo_from_file(self):
        """sffinfo_from_file should return file object with sffinfo output."""
        test_dir = os.path.dirname(os.path.dirname(os.path.abspath(__file__)))
        sff_fp = os.path.join(test_dir, 'data', 'test.sff')
        observed = sffinfo_from_file(sff_fp).read()
        self.assertEqual(observed, sffinfo_output)


sffinfo_output = '''Common Header:
  Magic Number:  0x2E736666
  Version:       0001
  Index Offset:  1504
  Index Length:  706
  # of Reads:    1
  Header Length: 440
  Key Length:    4
  # of Flows:    400
  Flowgram Code: 1
  Flow Chars:    TACGTACGTACGTACGTACGTACGTACGTACGTACGTACGTACGTACGTACGTACGTACGTACGTACGTACGTACGTACGTACGTACGTACGTACGTACGTACGTACGTACGTACGTACGTACGTACGTACGTACGTACGTACGTACGTACGTACGTACGTACGTACGTACGTACGTACGTACGTACGTACGTACGTACGTACGTACGTACGTACGTACGTACGTACGTACGTACGTACGTACGTACGTACGTACGTACGTACGTACGTACGTACGTACGTACGTACGTACGTACGTACGTACGTACGTACGTACGTACGTACGTACGTACGTACGTACGTACGTACGTACGTACGTACGTACGTACGTACGTACGTACGTACGTACGTACGTACGTACG
  Key Sequence:  TCAG

>FA6P1OK01CGMHQ
  Run Prefix:   R_2008_05_28_17_11_38_
  Region #:     1
  XY Location:  0892_1356

  Run Name:       R_2008_05_28_17_11_38_FLX02070135_adminrig_KnightLauber
  Analysis Name:  /data/2008_05_28/R_2008_05_28_17_11_38_FLX02070135_adminrig_KnightLauber/D_2008_05_28_21_13_06_FLX02070135_KnightLauber_FullAnalysisAmplicons
  Full Path:      /data/2008_05_28/R_2008_05_28_17_11_38_FLX02070135_adminrig_KnightLauber/D_2008_05_28_21_13_06_FLX02070135_KnightLauber_FullAnalysisAmplicons/../D_2008_05_29_13_52_01_FLX02070135_Knight_Lauber_jones_SignalProcessingAmplicons

  Read Header Len:  32
  Name Length:      14
  # of Bases:       77
  Clip Qual Left:   5
  Clip Qual Right:  52
  Clip Adap Left:   0
  Clip Adap Right:  0

Flowgram:	1.04	0.00	1.01	0.00	0.00	0.96	0.00	1.02	0.00	1.02	0.00	0.00	0.99	0.00	1.00	0.00	1.00	0.00	0.00	1.00	0.00	1.10	0.00	1.08	0.00	0.00	1.46	0.00	0.88	0.18	0.00	2.69	1.01	0.08	0.96	0.00	0.02	0.92	0.08	0.00	0.98	0.68	0.00	0.89	0.00	0.00	1.15	0.00	1.13	0.00	0.02	1.12	0.05	0.15	1.84	0.00	1.10	0.00	2.47	0.96	0.86	1.06	0.00	1.96	0.12	0.93	0.13	1.65	1.06	0.06	0.00	0.99	0.00	0.00	1.87	0.44	1.08	0.00	3.25	0.09	0.97	0.50	1.00	1.72	0.07	0.00	0.92	0.58	0.00	0.00	0.59	0.06	0.11	0.09	0.07	0.06	0.16	0.00	0.24	0.03	0.00	0.12	0.06	0.16	0.00	0.18	0.00	0.00	0.14	0.00	0.15	0.00	0.18	0.00	0.03	0.14	0.03	0.13	0.01	0.19	0.00	0.02	0.33	0.05	0.00	0.16	0.10	0.35	0.01	0.21	0.04	0.09	0.18	0.13	0.19	0.00	0.10	0.51	0.26	0.00	0.23	0.19	0.27	0.01	0.29	0.05	0.14	0.17	0.16	0.18	0.27	0.09	0.26	0.10	0.18	0.23	0.15	0.22	0.13	0.37	0.11	0.11	0.26	0.59	0.14	0.06	0.33	0.34	0.26	0.05	0.27	0.44	0.19	0.10	0.35	0.27	0.15	0.34	0.28	0.45	0.14	0.16	0.34	0.27	0.12	0.07	0.25	0.18	0.12	0.04	0.23	0.16	0.12	0.05	0.20	0.16	0.11	0.03	0.21	0.16	0.10	0.02	0.21	0.16	0.12	0.02	0.20	0.15	0.10	0.02	0.23	0.15	0.11	0.02	0.22	0.14	0.09	0.02	0.20	0.13	0.09	0.01	0.19	0.13	0.08	0.02	0.17	0.12	0.08	0.03	0.17	0.09	0.08	0.01	0.14	0.09	0.07	0.01	0.15	0.09	0.06	0.01	0.13	0.08	0.06	0.00	0.13	0.08	0.05	0.02	0.12	0.07	0.05	0.01	0.11	0.07	0.05	0.00	0.10	0.07	0.05	0.01	0.11	0.08	0.04	0.00	0.10	0.06	0.05	0.01	0.09	0.06	0.04	0.01	0.08	0.07	0.05	0.00	0.08	0.06	0.05	0.00	0.09	0.06	0.04	0.00	0.09	0.06	0.04	0.01	0.08	0.06	0.04	0.00	0.09	0.06	0.03	0.00	0.09	0.06	0.02	0.00	0.09	0.06	0.04	0.00	0.08	0.05	0.03	0.00	0.07	0.05	0.02	0.00	0.08	0.04	0.03	0.00	0.07	0.04	0.03	0.00	0.07	0.05	0.02	0.00	0.07	0.05	0.02	0.00	0.06	0.04	0.02	0.00	0.06	0.03	0.03	0.00	0.08	0.02	0.00	0.00	0.07	0.03	0.01	0.00	0.06	0.03	0.02	0.00	0.05	0.03	0.02	0.00	0.05	0.03	0.01	0.00	0.06	0.02	0.00	0.00	0.05	0.01	0.01	0.00	0.04	0.01	0.01	0.00	0.04	0.01	0.01	0.00	0.05	0.01	0.00	0.00	0.04	0.02	0.01	0.00	0.03	0.02	0.01	0.00	0.03	0.01	0.00	0.00	0.03	0.00	0.00	0.00	0.03	0.00	0.00	0.00	0.02	0.00
Flow Indexes:	1	3	6	8	10	13	15	17	20	22	24	27	29	32	32	32	33	35	38	41	42	44	47	49	52	55	55	57	59	59	60	61	62	64	64	66	68	68	69	72	75	75	77	79	79	79	81	82	83	84	84	87	88	91	102	126	130	138	140	145	153	157	161	164	166	171	175	179	183	187	191	195	199	203	211	215	219
Bases:	tcagATCTGAGCTGGGTCATAGCTGCCTCCGTAGGAGGTGCCTCCCTACGGCgcnnnannnnngnnnnnnnnnnnnn
Quality Scores:	32	32	32	32	32	32	32	32	32	32	32	25	25	21	21	21	28	32	32	31	30	30	32	32	32	33	31	25	18	18	20	18	32	30	28	23	22	22	24	28	18	19	18	16	16	16	17	18	13	17	27	21	20	21	0	0	0	17	0	0	0	0	0	17	0	0	0	0	0	0	0	0	0	0	0	0	0
'''

empty_sffinfo_output = '''Common Header:
  Magic Number:  0x2E736666
  Version:       0001
  Index Offset:  1504
  Index Length:  706
  # of Reads:    1
  Header Length: 440
  Key Length:    4
  # of Flows:    400
  Flowgram Code: 1
  Flow Chars:    TACGTACGTACGTACGTACGTACGTACGTACGTACGTACGTACGTACGTACGTACGTACGTACGTACGTACGTACGTACGTACGTACGTACGTACGTACGTACGTACGTACGTACGTACGTACGTACGTACGTACGTACGTACGTACGTACGTACGTACGTACGTACGTACGTACGTACGTACGTACGTACGTACGTACGTACGTACGTACGTACGTACGTACGTACGTACGTACGTACGTACGTACGTACGTACGTACGTACGTACGTACGTACGTACGTACGTACGTACGTACGTACGTACGTACGTACGTACGTACGTACGTACGTACGTACGTACGTACGTACGTACGTACGTACGTACGTACGTACGTACGTACGTACGTACGTACGTACGTACG
  Key Sequence:  TCAG
'''

if __name__ == '__main__':
    main()



#!/usr/bin/env python

"""Application controllers for pfold application package

Run in the same order as the order in this file(instructions from pfold author)
[fasta2col,findphyl,mltree,scfg]
"""
import os

from cogent.app.util import CommandLineApplication,\
    CommandLineAppResult, ResultPath

__author__ = "Shandy Wikman"
__copyright__ = "Copyright 2007-2016, The Cogent Project"
__contributors__ = ["Shandy Wikman"]
__license__ = "GPL"
__version__ = "1.9"
__maintainer__ = "Shandy Wikman"
__email__ = "ens01svn@cs.umu.se"
__status__ = "Development"  

#IMPORTANT!!!!
#pfold_path must be set manually to the bin dir in the install dir of pfold
if 'PFOLD_BIN_DIR' not in os.environ:
    raise RuntimeError, \
        "The pfold app controller requires PFOLD_BIN_DIR environment variable"
else:
    pfold_path = os.environ['PFOLD_BIN_DIR']

class fasta2col(CommandLineApplication):
    """Application controller for fasta2col in Pfold package"""

    _command = 'fasta2col'
    _input_handler = '_input_as_string'

    def _input_as_string(self,filename):
        """Overrides  _input_as_string in CommandLineApplication

        Input file need to be modified by sed"""
        sed = '| sed \'s/arbitrary/RNA/g\''
        data = '%s %s' % (filename,sed)
        return data
    
    def _input_as_lines(self,data):
        """Overrides _input_as_lines in CommandLineApplication

        Input file need to be modified by sed""" 
        filename = super(fasta2col,self)._input_as_lines(data)

        sed = '| sed \'s/arbitrary/RNA/g\''
        data = '%s %s' % (filename,sed)
        return data


class findphyl(CommandLineApplication):
    """Application controller for findphyl in Pfold package 

    Find the phylogeny of the sequences using the
    neighbour joining approach"""

    _command = 'findphyl'
    _input_handler = '_input_as_string'

    def _input_as_string(self,filename):
        """Overrides  _input_as_string in CommandLineApplication

        scfg.rate file need to be specified along with the input file""" 
        file = '%s%s' % (pfold_path,'scfg.rate')
        data = '%s %s' % (file,filename)
        return data
    
    def _input_as_lines(self,data):
        """Overrides _input_as_lines in CommandLineApplication

        scfg.rate file need to be specified along with the input file"""
        filename = super(findphyl,self)._input_as_lines(data)

        file = '%s%s' % (pfold_path,'scfg.rate')
        data = '%s %s' % (file,filename)
        return data



class mltree(CommandLineApplication):
    """Application controller for mltree in pfold package

    Performs a maximum likelihood estimate of the branch lengths"""

    _command = 'mltree'
    _input_handler = '_input_as_string'

    def _input_as_string(self,filename):
        """Overrides  _input_as_string in CommandLineApplication

        scfg.rate file need to be specified along with the input file"""
        file = '%s%s' % (pfold_path,'scfg.rate')
        data = '%s %s' % (file,filename)
        return data
    
    def _input_as_lines(self,data):
        """Overrides _input_as_lines in CommandLineApplication

        scfg.rate file need to be specified along with the input file"""
        filename = super(mltree,self)._input_as_lines(data)

        file = '%s%s' % (pfold_path,'scfg.rate')
        data = '%s %s' % (file,filename)
        return data


class scfg(CommandLineApplication):
    """Application controller for scfg in Pfold package

    Performs the analysis

    The file `article.grm' has the grammar and evolutionary model
    that is used for the analysis"""

    _command = 'scfg' 
    _input_handler = '_input_as_string'

    def _input_as_string(self,filename):
        """Overrides  _input_as_string in CommandLineApplication

        Additional input information about tree needed from article.grm file"""
        file = '%s %s%s' % ('--treeinfile',pfold_path,'article.grm')
        data = '%s %s' % (file,filename)
        return data
    
    def _input_as_lines(self,data):
        """Overrides _input_as_lines in CommandLineApplication

        Additional input information about tree needed from article.grm file"""
        filename = super(scfg,self)._input_as_lines(data)

        file = '%s %s%s' % ('--treeinfile',pfold_path,'article.grm')
        data = '%s %s' % (file,filename)
        return data




from cogent.util.unit_test import TestCase, main

from numpy import array, dot, diag, exp

import cogent.maths.matrix_exponentiation as cmme

from cogent.maths import matrix_exponential_integration as expm

__author__ = 'Ben Kaehler'
__copyright__ = "Copyright 2007-2014, The Cogent Project"
__credits__ = ['Ben Kaehler']
__license__ = "GPL"
__version__ = "1.9"
__maintainer__ = 'Ben Kaehler'
__email__ = 'benjamin.kaehler@anu.edu.au'
__status__ = "Production"

class TestIntegratingExponentiator(TestCase):
    def test_van_loan_integrating_exponentiator(self):
        """VanLoanIntegratingExponentiator should reproduce Felsenstein
        analytic result, should throw if we pass it a defected matrix and ask
        it to use CheckedExponentiator, will work with a defective matrix (that
        we can integrate by hand) if we use the default RobustExponentiator,
        and should work for different choices of R and exponentiatior."""
        # Result from Von Bing's R code
        result = 0.7295333
        q = array([[0.5, 0.2, 0.1, 0.2]]*4)
        for i in range(4):
            q[i, i] = 0.
            q[i, i] = -sum(q[i,])
        p0 = array([0.2, 0.3, 0.3, 0.2])

        I = expm.VanLoanIntegratingExponentiator(q, -diag(q))(1.0)
        self.assertFloatEqual(dot(p0, I), result)

        self.assertRaises(ArithmeticError,
                expm.VanLoanIntegratingExponentiator, 
                q, -diag(q), cmme.CheckedExponentiator)

        Q = array([[1., 1.], [0., 1.]])
        def integral(t):
            return array([[exp(t)-1., exp(t)*(t-1.)+1.], [0., exp(t)-1.]])
        
        self.assertFloatEqual(expm.VanLoanIntegratingExponentiator(Q)(1.),
            integral(1.))
        self.assertFloatEqual(expm.VanLoanIntegratingExponentiator(Q)(2.),
            integral(2.))

        R = array([[1.],[1.]])
        self.assertFloatEqual(expm.VanLoanIntegratingExponentiator(Q, R,
            cmme.TaylorExponentiator)(1.), dot(integral(1.), R))

    def test_von_bing_integrating_exponentiator(self):
        """VonBingIntegratingExponentiator should reproduce Felsenstein 
        analytic result, should throw if we pass it a defective matrix, and
        should match results obtained from VanLoanIntegratingExponentiator for
        a diagonisable matrix."""
        # Result from Von Bing's R code.
        result = 0.7295333
        q = array([[0.5, 0.2, 0.1, 0.2]]*4)
        for i in range(4):
            q[i, i] = 0.
            q[i, i] = -sum(q[i,])
        p0 = array([0.2, 0.3, 0.3, 0.2])

        I = expm.VonBingIntegratingExponentiator(q)(1.0)
        self.assertFloatEqual(dot(dot(p0, I), -diag(q)), result)

        self.assertRaises(ArithmeticError,
                expm.VonBingIntegratingExponentiator,
                array([[1., 1.], [0., 1.]]))

        p = array([[ 0.86758487,  0.05575623,  0.0196798 ,  0.0569791 ],
        [ 0.01827347,  0.93312148,  0.02109664,  0.02750842],
        [ 0.04782582,  0.1375742 ,  0.80046869,  0.01413129],
        [ 0.23022035,  0.22306947,  0.06995306,  0.47675713]])

        self.assertFloatEqual(expm.VonBingIntegratingExponentiator(p)(1.),
                expm.VanLoanIntegratingExponentiator(p,
                    exponentiator=cmme.FastExponentiator)(1.))
        self.assertFloatEqual(expm.VonBingIntegratingExponentiator(p)(2.),
                expm.VanLoanIntegratingExponentiator(p, 
                    exponentiator=cmme.FastExponentiator)(2.))



if __name__ == '__main__':
    main()
